/* -*- Mode: C++ -*- */

/* COPYING
 * Liquid Rendering Library
 * (c) 2009 by Henrik Theiling
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <locale.h>
#include <string.h>
#include <liquid/render++.h>

#define LB    "\x2028"
#define PARA  "\n"
#define LRE   "\x202A"
#define RLE   "\x202B"
#define PDF   "\x202C"
#define LRO   "\x202D"
#define RLO   "\x202E"
#define AL    "\x0643"
#define NSM   "\x0302"
#define EN    "0"
#define AN    "\x0661"
#define N     " "
#define L     "a"
#define R     "\x05D0"
#define ES    "-"
#define ET    "$"
#define CS    "."
#define LRM   "\x200E"
#define RLM   "\x200F"

#define KAF   "\x643"
#define LAM   "\x644"
#define MEEM  "\x645"

#define MEM_MAX 655360

//////////////////////////////////////////////////////////////////////

    static char mem[MEM_MAX];
    static char *mem_ptr= mem;

    void *stack_calloc (size_t n, size_t s)    /* usable as LIQ_CALLOC */
    {
        void *result= (void*)mem_ptr;
        size_t ns= n*s;
        if ((ns / s) != n ||                   /* integer overflow */
            mem_ptr + ns > mem + MEM_MAX)      /* memory overflow */
            return NULL;
        memset (mem_ptr, 0, ns);
        mem_ptr+= ns;
        return result;
    }

    void stack_free (void *p)                  /* usable as LIQ_FREE */
    {
        if (p == NULL) return;
        assert (p >= mem);
        assert (p <  mem + MEM_MAX);
        assert (p <= mem_ptr);
        mem_ptr= (char *)p;                    /* yes, it's that simple. */
    }

//////////////////////////////////////////////////////////////////////

int main (int, char **)
{
    liq_set_allocator (stack_calloc, stack_free);

    LiqRender render;
    render.set_font (liq_font_liqmean);
#ifdef LIQ_STR32

#if 0
    render.set_paragraph_level(0);
    render.set_text(LIQ_STR32("" L L R "."));
#elif 0
    render.set_paragraph_level(0);
    render.set_text(LIQ_STR32("" RLE L R R "." PDF));
#elif 0
    render.set_paragraph_level(0);
    render.set_text(LIQ_STR32("" L L "'" RLE L R R PDF ".'"));
#elif 0
    render.set_paragraph_level(1);
    render.set_text(LIQ_STR32("" R R R NSM NSM " '" LRE L NSM NSM L " \""
                                 RLE L R R PDF "\"" PDF "'?"));
#elif 0
    render.set_paragraph_level(0);
    render.set_text(LIQ_STR32("V_a\x2d9mmi"));
#elif 0
    render.set_paragraph_level(0);
    render.set_text(
        "ELעלALאל\nGidi Shalom-Bendor (גידי שלום-בן דור), Israel Gidali (ישראל גידלי)");
#elif 0
    //render.set_paragraph_level(1);
    render.set_text(
        "للكتابة ومنها... / ٠١٢٣٤٥٦٧٨٩\n"
        "میلیون تن (بین ۵۸٪ تا ۷۹٪) پارسی.\n"
    );
#elif 1
    //render.set_paragraph_level(1);
    render.set_text(
        LIQ_STR32("Hallo" LB
        "Welt!")
    );
#endif

#else /* LIQ_STR32 */

    render.set_text("Hello\vWorld!");

#endif /* LIQ_STR32 */

    render.bidi();
    render.bitmap();

    printf ("line_cnt=%d height=%d\n", render.get_line_count(), render.get_line_height());
    for (int l=0; l < render.get_line_count(); l++) {
        for (int y=0; y < render.get_line_height(); y++) {
            for (int x=0; x < render.get_line_width(l); x++) {
                printf ("%c", (render.get_column(l,x) & (1<< y)) ? '#' : '.');
            }
            printf ("\n");
        }
        printf ("\n");
    }

    return 0;
}
